#!/usr/bin/env python
# encoding: utf-8
#
# Copyright (c) 2015, Canonical Ltd.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A colorful ANSI color demonstration with Guacamole."""

from __future__ import absolute_import, print_function, unicode_literals

from guacamole import Command


class ANSIDemo(Command):

    """Demonstration of ANSI SGR codes."""

    def invoked(self, ctx):
        """Method called when the command is invoked."""
        if not ctx.ansi.is_enabled:
            print("You need color support to use this demo")
        else:
            print(ctx.ansi.cmd('erase_display'))
            self._demo_fg_color(ctx)
            self._demo_bg_color(ctx)
            self._demo_bg_indexed(ctx)
            self._demo_rgb(ctx)
            self._demo_style(ctx)

    def _demo_fg_color(self, ctx):
        self._header("Foreground Color", ctx)
        self._sub_header("Regular and Bright Foreground Sets", ctx)
        # Regular
        print(*[ctx.ansi('x' * (len(color) + 2), fg=color, bg='auto')
                for color in ctx.ansi.available_colors])
        print(*[ctx.ansi(' {} '.format(color.upper()), fg=color, bg='auto')
                for color in ctx.ansi.available_colors])
        print(*[ctx.ansi('x' * (len(color) + 2), fg=color, bg='auto')
                for color in ctx.ansi.available_colors])
        # Bright
        print(*[ctx.ansi('x' * (len(color) + 2),
                         fg='bright_{}'.format(color), bg='auto')
                for color in ctx.ansi.available_colors])
        print(*[ctx.ansi(' {} '.format(color.upper()),
                         fg='bright_{}'.format(color), bg='auto')
                for color in ctx.ansi.available_colors])
        print(*[ctx.ansi('x' * (len(color) + 2),
                         fg='bright_{}'.format(color), bg='auto')
                for color in ctx.ansi.available_colors])

    def _demo_bg_color(self, ctx):
        self._header("Background Color", ctx)
        self._sub_header("Regular and Bright Background Sets", ctx)
        # Regular
        print(*[ctx.ansi(' ' * (len(color) + 2), bg=color)
                for color in ctx.ansi.available_colors])
        print(*[ctx.ansi(' {} '.format(color.upper()), fg='auto', bg=color)
                for color in ctx.ansi.available_colors])
        print(*[ctx.ansi(' ' * (len(color) + 2), bg=color)
                for color in ctx.ansi.available_colors])
        # Bright
        print(*[ctx.ansi(' ' * (len(color) + 2), bg='bright_{}'.format(color))
                for color in ctx.ansi.available_colors])
        print(*[ctx.ansi(' {} '.format(color.upper()),
                         fg='auto', bg='bright_{}'.format(color))
                for color in ctx.ansi.available_colors])
        print(*[ctx.ansi(' ' * (len(color) + 2), bg='bright_{}'.format(color))
                for color in ctx.ansi.available_colors])

    def _demo_bg_indexed(self, ctx):
        self._header("Indexed 8-bit Background Color", ctx)
        self._sub_header("Regular and Bright Color Subsets", ctx)
        print(*(
            [ctx.ansi(' ' * 4, bg=i) for i in range(0x00, 0x07 + 1)]
            + [ctx.ansi(' ' * 4, bg=i) for i in range(0x08, 0x0F + 1)]))
        print(*(
            [ctx.ansi('{:02X}'.format(i).center(4), fg='auto', bg=i)
             for i in range(0x00, 0x07 + 1)]
            + [ctx.ansi('{:02X}'.format(i).center(4), fg='auto', bg=i)
               for i in range(0x08, 0x0F + 1)]))
        print(*(
            [ctx.ansi(' ' * 4, bg=i) for i in range(0x00, 0x07 + 1)]
            + [ctx.ansi(' ' * 4, bg=i) for i in range(0x08, 0x0F + 1)]))
        self._sub_header("6 * 6 * 6 RGB color subset", ctx)
        for y in range(6 * 3):
            print(*(
                [' ' * 5]
                + [ctx.ansi('{:02X}'.format(i).center(4), fg='auto', bg=i)
                   for i in range(0x10 + 6 * y, 0x10 + 6 * y + 6)]
                + [' ' * 6]
                + [ctx.ansi('{:02X}'.format(i).center(4), fg='auto', bg=i)
                    for i in range(0x7c + 6 * y, 0x7c + 6 * y + 6)]))
        self._sub_header("24 grayscale colors", ctx)
        print(
            '    ', *[ctx.ansi(' ' * 6, bg=i)
                      for i in range(0xE8, 0xF3 + 1)], sep='')
        print(
            '    ', *[ctx.ansi('{:02X}'.format(i).center(6), fg='auto', bg=i)
                      for i in range(0xE8, 0xF3 + 1)], sep='')
        print(
            '    ', *[ctx.ansi(' ' * 6, bg=i)
                      for i in range(0xE8, 0xF3 + 1)], sep='')
        print(
            '    ', *[ctx.ansi(' ' * 6, bg=i)
                      for i in range(0xF4, 0xFF + 1)], sep='')
        print(
            '    ', *[ctx.ansi('{:02X}'.format(i).center(6), fg='auto', bg=i)
                      for i in range(0xF4, 0xFF + 1)], sep='')
        print(
            '    ', *[ctx.ansi(' ' * 6, bg=i)
                      for i in range(0xF4, 0xFF + 1)], sep='')

    def _demo_rgb(self, ctx):
        self._header("24 bit RGB Color", ctx)
        self._sub_header("The bar below only displays 80 unique colors", ctx)
        cols = 80
        for y in range(3):
            print(*[ctx.ansi(' ', fg='auto', bg=hsv(360.0 / cols * i, 1, 1))
                    for i in range(cols)], sep='')

    def _demo_style(self, ctx):
        self._header("Text style", ctx)
        styles = ctx.ansi.available_styles
        print(*[ctx.ansi(style, style=style) for style in styles])

    def _header(self, text, ctx):
        print(
            ctx.ansi(' ' * (40 - len(text) // 2), bg=0xE2),
            ctx.ansi(text, fg=0x10, bg=0xE2),
            ctx.ansi(' ' * (40 - len(text) // 2), bg=0xE2),
            sep='')

    def _sub_header(self, text, ctx):
        print(
            ctx.ansi(text, fg=0xFF, bg=0x10),
            ctx.ansi(' ' * (80 - len(text)), bg=0x10),
            sep='')


def hsv(h, s, v):
    """Convert HSV (hue, saturation, value) to RGB."""
    if 360 < h < 0:
        raise ValueError("h out of range: {}".format(h))
    if 1 < s < 0:
        raise ValueError("s out of range: {}".format(h))
    if 1 < v < 0:
        raise ValueError("v out of range: {}".format(h))
    c = v * s  # chroma
    h1 = h / 60
    x = c * (1 - abs(h1 % 2 - 1))
    if 0 <= h1 < 1:
        r1, g1, b1 = (c, x, 0)
    elif 1 <= h1 < 2:
        r1, g1, b1 = (x, c, 0)
    elif 2 <= h1 < 3:
        r1, g1, b1 = (0, c, x)
    elif 3 <= h1 < 4:
        r1, g1, b1 = (0, x, c)
    elif 4 <= h1 < 5:
        r1, g1, b1 = (x, 0, c)
    elif 5 <= h1 < 6:
        r1, g1, b1 = (c, 0, x)
    m = v - c
    r, g, b = r1 + m, g1 + m, b1 + m
    return int(r * 255), int(g * 255), int(b * 255)


if __name__ == '__main__':
    ANSIDemo().main()



# encoding: utf-8
# This file is part of Guacamole.
#
# Copyright 2012-2015 Canonical Ltd.
# Written by:
#   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
#
# Guacamole is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3,
# as published by the Free Software Foundation.
#
# Guacamole is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Guacamole.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the cmdtree module."""

from __future__ import absolute_import, print_function, unicode_literals

import unittest

from guacamole.core import Bowl
from guacamole.ingredients.cmdtree import CommandTreeBuilder
from guacamole.recipes.cmd import Command


class _sub(Command):
    spices = ('mustard',)


class _cmd(Command):
    spices = ('salt', 'pepper')
    sub_commands = (('sub', _sub),)


class CommandTreeBuilderTests(unittest.TestCase):

    """Tests for the CommandTreeBuilder class."""

    def setUp(self):
        """Common initialization method."""
        self.bowl = Bowl([CommandTreeBuilder(_cmd())])
        self.bowl.eat()

    def test_build_command_tree(self):
        """check if a correct command tree is built."""
        cmd_obj = self.bowl.context.cmd_tree[1]
        sub_obj = self.bowl.context.cmd_tree[2][0][1]
        self.assertIsInstance(cmd_obj, _cmd)
        self.assertIsInstance(sub_obj, _sub)
        self.assertEqual(
            self.bowl.context.cmd_tree,
            (None, cmd_obj, (('sub', sub_obj, ()),)))

    def test_collect_spices(self):
        """check if spices are collected from top-level command only."""
        self.assertTrue(self.bowl.has_spice('salt'))
        self.assertTrue(self.bowl.has_spice('pepper'))
        self.assertFalse(self.bowl.has_spice('mustard'))



# encoding: utf-8
# This file is part of Guacamole.
#
# Copyright 2012-2015 Canonical Ltd.
# Written by:
#   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
#
# Guacamole is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3,
# as published by the Free Software Foundation.
#
# Guacamole is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Guacamole.  If not, see <http://www.gnu.org/licenses/>.

"""
The essence of guacamole.

This module defines the three essential core classes: :class:`Ingredient`,
:class:`Bowl`, :class:`Context`. All of those have stable APIs.
"""

from __future__ import absolute_import, print_function, unicode_literals

import logging
import sys

__all__ = (
    'Bowl',
    'Context',
    'Ingredient',
)


_logger = logging.getLogger('guacamole')


class Ingredient(object):

    """
    Part of guacamole.

    Ingredients are a mechanism for inserting functionality into Guacamole.
    The sequence of calls to ingredient methods is as follows:

    - :meth:`added()`

    The added method is where an ingredient can advertise itself to other
    ingredients that it explicitly collaborates with.

    - :meth:`preparse()`

    The preparse method is where ingredients can have a peek at the command
    line arguments. This can serve to optimize further actions. Essentially
    guacamole allows applications to parse arguments twice and limit the
    actions needed to do that correctly to the essential minimum required.

    - :meth:`early_init()`

    The early initialization method can be used to do additional
    initialization. It can take advantage of the fact that the whole command
    line arguments are now known and may have been analyzed further by the
    preparse method.

    - :meth:`parse()`

    The parse method is where applications are expected to fully understand
    command line arguments. This method can abort subsequent execution if
    arguments are wrong in in some way. After parsing command line arguments
    the application should be ready for execution.

    - :meth:`late_init()`

    The late initialization method mimics the early initialization method but
    is called after parsing all of the command line arguments. Again, it can be
    used to prepare addiotional resources necessary for a given application.

    - :meth:`dispatch()`

    The dispatch method is where applications execute the bulk of their
    actions.  Dispatching is typically done with one of the standard
    ingredients which will locate the appropriate method to call into the
    application.

    Depending on the outcome of the dispatch (if an exception is raised or not)
    one of :meth:`dispatch_succeeded()`` or :meth:`dispatch_failed()` is
    called.

    - :meth:`shutdown()`

    This is the last method called on all ingredients.

    Each of those methods is called with a context argument
    (:class:`Context:`).  A context is a free-for-all environment where
    ingredients can pass data around. There is no name-spacing. Ingredients
    should advertise what they do with the context and what to expect.
    """

    def __str__(self):
        """
        Get the string representation of this ingredient.

        The string method just returns the class name. Since the ingredient is
        an implemenetation detail it does not have anything that applications
        should show to the user.
        """
        return self.__class__.__name__

    def added(self, context):
        """Ingredient method called before anything else."""

    def build_early_parser(self, context):
        """Ingredient method called to build the early parser."""

    def preparse(self, context):
        """Ingredient method called to pre-parse command line aruments."""

    def early_init(self, context):
        """Ingredient method for early initialization."""

    def build_parser(self, context):
        """Ingredient method called to build the full parser."""

    def parse(self, context):
        """Ingredient method called to parse command line arguments."""

    def late_init(self, context):
        """Ingredient method for late initialization."""

    def dispatch(self, context):
        """
        Ingredient method for dispatching (execution).

        .. note::
            The first ingredient that implements this method and returns
            something other than None will stop command dispatch!
        """

    def dispatch_succeeded(self, context):
        """Ingredient method called when dispatching is correct."""

    def dispatch_failed(self, context):
        """Ingredient method called when dispatching fails."""

    def shutdown(self, context):
        """Ingredient method called after all other methods."""


class Context(object):

    """
    Context for making guacamole with ingredients.

    A context object is created and maintained throughout the life-cycle of an
    executing tool. A context is passed as argument to all ingredient methods.

    Since context has no fixed API anything can be stored and loaded.
    Particular ingredients document how they use the context object.
    """

    def __repr__(self):
        """
        Get a debugging string representation of the context.

        The debugging representation shows all of the *names* of objects added
        to the context by various ingredients. Since the actual object can have
        large and complex debugging representation containing that
        representation was considered as a step against understanding what is
        in the context.
        """
        return "<Context {{{}}}>".format(
            ', '.join(sorted(self.__dict__.keys())))


class Bowl(object):

    """
    A vessel for preparing guacamole out of ingredients.

    .. note::
        Each Bowl is single-use. If you eat it you need to get another one as
        this one is dirty and cannot be reused.
    """

    def __init__(self, ingredients):
        """Prepare a guacamole out of given ingredients."""
        self.ingredients = ingredients
        self.context = Context()
        self.context.bowl = self
        self.context.spices = set()

    def add_spice(self, spice):
        """
        Add a single spice the bowl.
        """
        self.context.spices.add(spice)

    def has_spice(self, spice):
        """
        Check if a given spice is being used.

        This method can be used to construct checks if an optional ingredient
        feature should be enabled or not. Spices are simply strings that
        describe optional features.
        """
        return spice in self.context.spices

    def eat(self, argv=None):
        """
        Eat the guacamole.

        :param argv:
            Command line arguments or None. None means that sys.argv is used
        :return:
            Whatever is returned by the first ingredient that agrees to perform
            the command dispatch.

        The eat method is called to run the application, as if it was invoked
        from command line directly.
        """
        # The setup phase, here KeyboardInterrupt is a silent sign to exit the
        # application. Any error that happens here will result in a raw
        # backtrace being printed to the user.
        try:
            self.context.argv = argv
            self._added()
            self._build_early_parser()
            self._preparse()
            self._early_init()
            self._build_parser()
            self._parse()
            self._late_init()
        except KeyboardInterrupt:
            self._shutdown()
            return
        # The execution phase. Here we differentiate SystemExit from all other
        # exceptions. SystemExit is just re-raised as that's what any piece of
        # code can raise to ask to exit the currently running application.  All
        # other exceptions are recorded in the context and the failure-path of
        # the dispatch is followed. In other case, when there are no
        # exceptions, the success-path is followed. In both cases, ingredients
        # are shut down.
        try:
            return self._dispatch()
        except SystemExit:
            raise
        except BaseException:
            (self.context.exc_type, self.context.exc_value,
             self.context.traceback) = sys.exc_info()
            self._dispatch_failed()
        else:
            self._dispatch_succeeded()
        finally:
            self._shutdown()

    def _added(self):
        """Run the added() method on all ingredients."""
        for ingredient in self.ingredients:
            ingredient.added(self.context)

    def _build_early_parser(self):
        """Run build_early_parser() method on all ingredients."""
        for ingredient in self.ingredients:
            ingredient.build_early_parser(self.context)

    def _preparse(self):
        """Run the peparse() method on all ingredients."""
        for ingredient in self.ingredients:
            ingredient.preparse(self.context)

    def _early_init(self):
        """Run the early_init() method on all ingredients."""
        for ingredient in self.ingredients:
            ingredient.early_init(self.context)

    def _build_parser(self):
        """Run build_parser() method on all ingredients."""
        for ingredient in self.ingredients:
            ingredient.build_parser(self.context)

    def _parse(self):
        """Run the parse() method on all ingredients."""
        for ingredient in self.ingredients:
            ingredient.parse(self.context)

    def _late_init(self):
        """Run the late_init() method on all ingredients."""
        for ingredient in self.ingredients:
            ingredient.late_init(self.context)

    def _dispatch(self):
        """Run the dispatch() method on all ingredients."""
        for ingredient in self.ingredients:
            result = ingredient.dispatch(self.context)
            if result is not None:
                return result

    def _dispatch_succeeded(self):
        """Run the dispatch_succeeded() method on all ingredients."""
        for ingredient in self.ingredients:
            ingredient.dispatch_succeeded(self.context)

    def _dispatch_failed(self):
        """Run the dispatch_failed() method on all ingredients."""
        for ingredient in self.ingredients:
            ingredient.dispatch_failed(self.context)

    def _shutdown(self):
        """Run the shutdown() method on all ingredients."""
        for ingredient in self.ingredients:
            ingredient.shutdown(self.context)



# encoding: utf-8
# This file is part of Guacamole.
#
# Copyright 2012-2015 Canonical Ltd.
# Written by:
#   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
#
# Guacamole is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3,
# as published by the Free Software Foundation.
#
# Guacamole is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Guacamole.  If not, see <http://www.gnu.org/licenses/>.

"""
Recipe for using guacamole to run commands.

This module contains sock recipes for guacamole. Stock recipes allow
application developers to use use simple-to-understand design patterns to get
predictable runtime behiavior.

Currently, guacamole ships with two such recipes, for creating simple commands
and for creating hierarhical command groups. They are captured by the
:class:`Command` and :class:`Group` classes respectively.
"""

from __future__ import absolute_import, print_function, unicode_literals

import gettext
import inspect
import logging

from guacamole.ingredients import ansi
from guacamole.ingredients import argparse
from guacamole.ingredients import cmdtree
from guacamole.ingredients import crash
from guacamole.recipes import Recipe


__all__ = (
    'Command',
    'CommandRecipe',
)


_logger = logging.getLogger('guacamole')


class Command(object):

    """
    A single-purpose command.

    Single purpose commands are the most commonly known tools in command line
    environments. Tools such as ``ls``, ``mkdir`` or ``vim`` all fall in this
    class. A command is essentially a named action that can be invoked from the
    terminal emulator or other command line environment specific to a given
    operating system.

    To create a new command simply create a custom class and override the
    :meth:`invoked()` method. Put all of your custom code there. If you want to
    interact with command line arguments then please also override the
    :meth:`register_arguments()` method.

    Have a look at example applications for details of how to do this. You can
    use them as a starting point for your own application as they are licensed
    very liberally.
    """

    def __repr__(self):
        """Get the debugging representation of a command."""
        return "<{}>".format(self.__class__.__name__)

    def invoked(self, context):
        """
        Callback called when the command gets invoked.

        :param context:
            The guacamole context object.
        :returns:
            The return value is returned by the executable. It should be an
            integer between 0 and 255. Other values are will likely won't work
            at all.

        The context argument can be used to access command line arguments and
        other information that guacamole provides.
        """
        if not self.get_sub_commands():
            _logger.warning(
                "Command %r doesn't override Command.invoked()", self)

    def register_arguments(self, parser):
        """
        Callback called to register command-specific arguments.

        :param parser:
            Argument parser (from :mod:`argparse`) specific to this command.
        """

    def get_app_vendor(self):
        """
        Get the name of the application vendor.

        The name should be a human readable name, like ``"Joe Developer"`` or
        ``"Big Corporation Ltd."``

        .. note::
            Application vendor name is looked up using the ``app_vendor``
            attribute.
        """
        try:
            return self.app_vendor
        except AttributeError:
            pass

    def get_app_name(self):
        """
        Get the name of the application.

        .. note::
            Application name is looked up using the ``app_name`` attribute.

        Application name differs from executable name. The executable might be
        called ``my-app`` or ``myapp`` while the application might be called
        ``My Application``.
        """
        try:
            return self.app_name
        except AttributeError:
            pass

    def get_app_id(self):
        """
        Get the identifier of the application.

        .. note::
            Application identifier is looked up using the ``app_id`` attribute.

        The syntax of a valid command identifier is ``REVERSE-DNS-NAME:ID``.
        For example, ``"com.example.product:command"``. This identifier must
        not contain characters that are hostile to the file systems. It's best
        to stick to ASCII characters and digits.

        On *Mac OS X* this will be used as a directory name rooted in
        ``~/Library/Preferences/``. On Linux and other freedesktop.org-based
        systems this will be used as directory name rooted in
        ``$XDG_CONFIG_HOME`` and ``$XDG_CACHE_HOME``. On Windows it will be
        used as a directory name rooted in the per-user ``AppData`` folder.

        .. note::
            If this method returns None then logging and configuration services
            are disabled. It is strongly recommended to implement this method
            and return a correct value as it enhances application behavior.
        """
        try:
            return self.app_id
        except AttributeError:
            pass

    def get_cmd_name(self):
        """
        Get the name of the application executable.

        .. note::
            If this method returns None then the executable name is guessed
            from ``sys.argv[0]``.
        """
        try:
            return self.name
        except AttributeError:
            pass

    def get_cmd_version(self):
        """
        Get the version reported by this executable.

        .. note::
            If this method returns None then the ``--version`` option
            is disabled.
        """
        try:
            return self.version
        except AttributeError:
            pass

    def get_cmd_usage(self):
        """
        Get the usage string associated with this command.

        :returns:
            ``self.usage``, if defined
        :returns:
            None, otherwise

        The usage string typically contains the list of available, abbreviated
        options, mandatory arguments and other arguments. Its purpose is to
        quickly inform the user on the basic syntax used by the command.

        It is perfectly fine not to customize this method as the default is to
        compute an appropriate usage string out of all the arguments.  Consider
        implementing this method in a customized way if your command has highly
        complicated syntax and you want to provide an alternative, more terse
        usage string instead.
        """
        try:
            return self.usage
        except AttributeError:
            pass

    def get_cmd_help(self):
        """
        Get the single-line help of this command.

        :returns:
            ``self.help``, if defined
        :returns:
            The first line of the docstring, without the trailing dot, if
            present.
        :returns:
            None, otherwise
        """
        try:
            return self.help
        except AttributeError:
            pass
        try:
            return get_localized_docstring(
                self, self.get_gettext_domain()
            ).splitlines()[0].rstrip('.').lower()
        except (AttributeError, IndexError, ValueError):
            pass

    def get_cmd_description(self):
        """
        Get the leading, multi-line description of this command.

        :returns:
            ``self.description``, if defined
        :returns:
            A substring of the class docstring between the first line (which
            is discarded) and the string ``@EPILOG@``, if present, or the end
            of the docstring, if any
        :returns:
            None, otherwise

        The description string will be displayed after the usage string but
        before any of the detailed argument descriptions.

        Please consider following good practice by keeping the description line
        short enough not to require scrolling but useful enough to provide
        additional information that cannot be inferred from the name of the
        command or other arguments. Stating the purpose of the command is
        highly recommended.
        """
        try:
            return self.description
        except AttributeError:
            pass
        try:
            return '\n'.join(
                get_localized_docstring(
                    self, self.get_gettext_domain()
                ).splitlines()[1:]
            ).split('@EPILOG@', 1)[0].strip()
        except (AttributeError, IndexError, ValueError):
            pass

    def get_cmd_epilog(self):
        """
        Get the trailing, multi-line description of this command.

        :returns:
            ``self.epilog``, if defined
        :returns:
            A substring of the class docstring between the string ``@EPILOG``
            and the end of the docstring, if defined
        :returns:
            None, otherwise

        The epilog is similar to the description string but it is instead
        printed after the section containing detailed descriptions of all of
        the command line arguments.

        Please consider following good practice by providing additional details
        about how the command can be used, perhaps an example or a reference to
        means of finding additional documentation.
        """
        try:
            return self.source.epilog
        except AttributeError:
            pass
        try:
            return '\n'.join(
                get_localized_docstring(
                    self, self.get_gettext_domain()
                ).splitlines()[1:]
            ).split('@EPILOG@', 1)[1].strip()
        except (AttributeError, IndexError, ValueError):
            pass

    def get_gettext_domain(self):
        """
        Get the gettext translation domain associated with this command.

        The value returned will be used to select translations to global calls
        to gettext() and ngettext() everywhere in python.

        .. note::
            If this method returns None then all i18n services are disabled.
        """
        try:
            return self.gettext_domain
        except AttributeError:
            pass

    def get_locale_dir(self):
        """
        Get the path of the gettext translation catalogs for this command.

        This value is used to bind the domain returned by
        :meth:`get_gettext_domain()` to a specific directory.

        .. note::
            If this method returns None then standard, system-wide locations
            are used (on compatibles systems). In practical terms, on Windows,
            you may need to use it to have access to localization data.
        """
        try:
            return self.locale_dir
        except AttributeError:
            pass

    def get_sub_commands(self):
        """
        Get a list of sub-commands of this command.

        :returns:
            ``self.sub_commands``, if defined. This is a sequence of pairs
            ``(name, cls)`` where ``name`` is the name of the sub command and
            ``cls`` is a command class (not an object). The ``name`` can be
            None if the command has a version of :meth:`get_cmd_name()` that
            returns an useful value.
        :returns:
            An empty tuple otherwise

        Applications can create hierarchical commands by defining the
        ``sub_commands`` attribute. Many developers are familiar with nested
        commands, for example ``git commit`` is a sub-command of the ``git``
        command. All commands can be nested this way.
        """
        try:
            return self.sub_commands
        except AttributeError:
            return ()

    def get_cmd_spices(self):
        """
        Get a list of spices requested by this command.

        Feature flags are a mechanism that allows application developers to
        control ingredients (switch them on or off) as well as to control how
        some ingredients behave.

        :returns:
            ``self.spices``, if defined. This should be a set of strings.  Each
            string represents as single flag. Ingredients should document the
            set of flags they understand and use.
        :returns:
            An empty set otherwise

        Some flags have a generic meaning, you can scope a flag to a given
        ingredient using the ``name:`` prefix where the name is the name of the
        ingredient.
        """
        try:
            spices = self.spices
        except AttributeError:
            spices = set()
        return spices

    def main(self, argv=None, exit=True):
        """
        Shortcut for running a command.

        See :meth:`guacamole.recipes.Recipe.main()` for details.
        """
        return CommandRecipe(self).main(argv, exit)


def get_localized_docstring(obj, domain):
    """Get a cleaned-up, localized copy of docstring of this class."""
    if obj.__class__.__doc__ is not None:
        return inspect.cleandoc(
            gettext.dgettext(domain, obj.__class__.__doc__))


class CommandRecipe(Recipe):

    """A recipe for using commands."""

    def __init__(self, command):
        """Initialize a recipe for working with a specific command."""
        self.command = command

    def get_ingredients(self):
        """Get a list of ingredients for guacamole."""
        return [
            ansi.ANSIIngredient(),
            cmdtree.CommandTreeBuilder(self.command),
            cmdtree.CommandTreeDispatcher(),
            argparse.AutocompleteIngredient(),
            argparse.ParserIngredient(),
            crash.VerboseCrashHandler(),
        ]



# encoding: utf-8
# This file is part of Guacamole.
#
# Copyright 2012-2015 Canonical Ltd.
# Written by:
#   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
#
# Guacamole is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3,
# as published by the Free Software Foundation.
#
# Guacamole is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Guacamole.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for guacamole core."""

from __future__ import absolute_import, print_function, unicode_literals

import unittest

from guacamole.core import Bowl


class TestBowl(unittest.TestCase):

    """Tests for the Bowl class."""

    def setUp(self):
        """Common initialization code."""
        self.bowl = Bowl([])

    def test_spices(self):
        """Bowl.add_spice() and Bowl.has_spice() work as expected."""
        self.assertFalse(self.bowl.has_spice("salt"))
        self.bowl.add_spice('salt')
        self.assertTrue(self.bowl.has_spice("salt"))



# encoding: utf-8
# This file is part of Guacamole.
#
# Copyright 2012-2015 Canonical Ltd.
# Written by:
#   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
#
# Guacamole is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3,
# as published by the Free Software Foundation.
#
# Guacamole is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Guacamole.  If not, see <http://www.gnu.org/licenses/>.

"""Ingredient for reacting to application crashes."""

from __future__ import absolute_import, print_function, unicode_literals

import traceback

from guacamole.core import Ingredient


class VerboseCrashHandler(Ingredient):

    """
    Ingredient for reacting to crashes with a traceback.

    You can add this ingredient into your recipe to react to application
    crashes. It will simply print the exception, as stored in
    ``context.exc_type``, ``context.exc_value`` and ``context.traceback`` and
    raise SystemExit(1).
    """

    def dispatch_failed(self, context):
        """Print the unhandled exception and exit the application."""
        traceback.print_exception(
            context.exc_type, context.exc_value, context.traceback)
        raise SystemExit(1)



# encoding: utf-8
# This file is part of Guacamole.
#
# Copyright 2012-2015 Canonical Ltd.
# Written by:
#   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
#
# Guacamole is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3,
# as published by the Free Software Foundation.
#
# Guacamole is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Guacamole.  If not, see <http://www.gnu.org/licenses/>.

"""Ingredients for using ANSI command sequences."""

from __future__ import absolute_import, print_function, unicode_literals

import sys

from guacamole.core import Ingredient


class ANSI(object):

    """
    Numerous ANSI constants.

    .. seealso::

        Original specification in the `Standard ECMA 48
        <http://www.ecma-international.org/publications/
        files/ECMA-ST/Ecma-048.pdf>`_, page 61 (75th page of
        the PDF).

        Wikipedia article about `ANSI escape code
        <http://en.wikipedia.org/wiki/ANSI_escape_code>`_.

    .. attribute:: cmd_erase_display
        Command for erasing the whole display

    .. attribute:: cmd_erase_line
        Command for erasing the current line

    .. attribute:: cmd_sgr_reset_all:
        Command for resetting all SGR attributes

    .. attribute:: sgr_reset_all:
        SGR code for resetting all attributes

    .. attribute:: sgr_bold:

        Causes text to be rendered with bold face font or, alternatively,
        to be rendered with bright color variant. This code is widely
        supported on Linux. It is not supported on Windows.

    .. attribute:: sgr_bright:
        Alternate spelling of :attr:`sgr_bold`.

    .. attribute:: sgr_faint:
        SGR code that activates faint color subset

    .. attribute:: srg_dim:
        Alternate spelling of ``sgr_faint``

    .. attribute:: srg_italic:
        SGR code that activates italic font face

    .. attribute:: sgr_underline:
        SGR code that activates underline mode

    .. attribute:: sgr_blink_slow:
        SGR code that activates slow blinking of characters

    .. attribute:: sgr_blink_fast:
        SGR code that activates fast blinking of characters

    .. attribute:: sgr_reverse:
        SGR code that activates reverse-video mode

    .. attribute:: sgr_double_underline:
        SGR code that activates double-underline mode
    """

    cmd_erase_display = '\033[2J'
    cmd_erase_line = '\033[K'
    cmd_sgr_reset_all = '\033[0m'

    @staticmethod
    def cmd_sgr(sgr_list):
        """Get a SGR (Set Graphics Rendition) code."""
        return '\033[{}m'.format(';'.join(sgr_list))

    # SGR 0-9: text attribute control

    sgr_reset_all = '0'
    sgr_bold = '1'
    sgr_bright = '1'
    sgr_faint = sgr_dim = '2'
    sgr_italic = '3'
    sgr_underline = '4'
    sgr_blink_slow = '5'
    sgr_blink_fast = '6'
    sgr_reverse = '7'
    sgr_concealed = '8'
    sgr_crossed = '9'

    # SGR 10-20: font set control (not implemented)

    sgr_font_default = '10'
    sgr_font_alt1 = '11'
    sgr_font_alt2 = '12'
    sgr_font_alt3 = '13'
    sgr_font_alt4 = '14'
    sgr_font_alt5 = '15'
    sgr_font_alt6 = '16'
    sgr_font_alt7 = '17'
    sgr_font_alt8 = '18'
    sgr_font_alt9 = '19'
    sgr_font_fraktur = '20'

    # SGR 21-29: text attribute control (cont.)

    sgr_double_underline = '21'
    sgr_normal = '22'  # undo sgr_bold and sgr_dim
    sgr_not_italic = '23'  # also undoes Fraktur
    sgr_not_underline = '24'
    sgr_steady = '25'  # undo sgr_blink_{slow,fast}
    # 26 - reserved for proportional spacing
    sgr_positive = '27'  # undo sgr_reverse
    sgr_reveal = '28'  # undo sgr_concealed
    sgr_not_crossed = '29'  # undo sgr_crossed

    # SGR 30-39: foreground color control

    sgr_fg_black = '30'
    sgr_fg_red = '31'
    sgr_fg_green = '32'
    sgr_fg_yellow = '33'
    sgr_fg_blue = '34'
    sgr_fg_magenta = '35'
    sgr_fg_cyan = '36'
    sgr_fg_white = '37'

    @staticmethod
    def sgr_fg_rgb(r, g, b):
        """Get SGR (Set Graphics Rendition) foreground RGB color."""
        return '38;2;{};{};{}'.format(r, g, b)

    @staticmethod
    def sgr_fg_indexed(i):
        """Get SGR (Set Graphics Rendition) foreground indexed color."""
        return '38;5;{}'.format(i)

    sgr_fg_default = '39'

    # SGR 40-49: background color control

    sgr_bg_black = '40'
    sgr_bg_red = '41'
    sgr_bg_green = '42'
    sgr_bg_yellow = '43'
    sgr_bg_blue = '44'
    sgr_bg_magenta = '45'
    sgr_bg_cyan = '46'
    sgr_bg_white = '47'

    @staticmethod
    def sgr_bg_rgb(r, g, b):
        """Get SGR (Set Graphics Rendition) background RGB color."""
        return '48;2;{};{};{}'.format(r, g, b)

    @staticmethod
    def sgr_bg_indexed(i):
        """Get SGR (Set Graphics Rendition) background indexed color."""
        return '48;5;{}'.format(i)

    sgr_bg_default = '49'

    # SGR 90-97: high-intensity foreground color control

    sgr_fg_bright_black = '90'
    sgr_fg_bright_red = '91'
    sgr_fg_bright_green = '92'
    sgr_fg_bright_yellow = '93'
    sgr_fg_bright_blue = '94'
    sgr_fg_bright_magenta = '95'
    sgr_fg_bright_cyan = '96'
    sgr_fg_bright_white = '97'

    # SGR 100-107: high-intensity background color control

    sgr_bg_bright_black = '100'
    sgr_bg_bright_red = '101'
    sgr_bg_bright_green = '102'
    sgr_bg_bright_yellow = '103'
    sgr_bg_bright_blue = '104'
    sgr_bg_bright_magenta = '105'
    sgr_bg_bright_cyan = '106'
    sgr_bg_bright_white = '107'


def ansi_cmd(cmd, *args):
    """Get ANSI command code by name."""
    try:
        obj = getattr(ANSI, str('cmd_{}'.format(cmd)))
    except AttributeError:
        raise ValueError(
            "incorrect command: {!r}".format(cmd))
    if isinstance(obj, type("")):
        return obj
    else:
        return obj(*args)


class _Visible:

    black = str('white')
    red = str('black')
    green = str('black')
    yellow = str('black')
    blue = str('black')
    magenta = str('black')
    cyan = str('black')
    white = str('black')

    bright_black = str('bright_white')
    bright_red = str('bright_black')
    bright_green = str('bright_black')
    bright_yellow = str('bright_black')
    bright_blue = str('bright_black')
    bright_magenta = str('bright_black')
    bright_cyan = str('bright_black')
    bright_white = str('bright_black')

    default = str('default')


def get_visible_color(color):
    """Get the visible counter-color."""
    if isinstance(color, (str, type(""))):
        try:
            return getattr(_Visible, str('{}'.format(color)))
        except AttributeError:
            raise ValueError("incorrect color: {!r}".format(color))
    elif isinstance(color, tuple):
        return (0x80 ^ color[0], 0x80 ^ color[1], 0x80 ^ color[2])
    elif isinstance(color, int):
        if 0 <= color <= 0x07:
            index = color
            return 0xFF if index == 0 else 0xE8
        elif 0x08 <= color <= 0x0F:
            index = color - 0x08
            return 0xFF if index == 0 else 0xE8
        elif 0x10 <= color <= 0xE7:
            index = color - 0x10
            if 0 <= index % 36 < 18:
                return 0xFF
            else:
                return 0x10
        elif 0xE8 <= color <= 0xFF:
            index = color - 0x0E8
            return 0xFF if 0 <= index < 12 else 0xE8
    else:
        raise ValueError("incorrect color: {!r}".format(color))


def get_intensity(r, g, b):
    """Get the gray level intensity of the given rgb triplet."""
    return int(round(255 * (0.3 * r + 0.59 * g + 0.11 * b)))


def ansi_sgr(text, fg=None, bg=None, style=None, reset=True, **sgr):
    """
    Apply desired SGR commands to given text.

    :param text:
        Text or anything convertible to text
    :param fg:
        (optional) Foreground color. Choose one of
        ``black``, ``red``, ``green``, ``yellow``, ``blue``, ``magenta``
        ``cyan`` or ``white``. Note that the ``bright`` *SGR* impacts
        effective color in most implementations.

    """
    # Ensure that text is really a string
    text = str(text)
    # NOTE: SGR stands for "set graphics rendition"
    sgr_list = []  # List of SGR codes
    # Load SGR code associated with desired foreground color
    if isinstance(fg, (str, type(""))):
        try:
            sgr_code = getattr(ANSI, str('sgr_fg_{}'.format(fg)))
        except AttributeError:
            raise ValueError("incorrect foreground color: {!r}".format(fg))
        else:
            sgr_list.append(sgr_code)
    elif isinstance(fg, tuple):
        sgr_code = ANSI.sgr_fg_rgb(*fg)
        sgr_list.append(sgr_code)
    elif isinstance(fg, int):
        sgr_code = ANSI.sgr_fg_indexed(fg)
        sgr_list.append(sgr_code)
    elif fg is None:
        pass
    else:
        raise ValueError("incorrect foreground color: {!r}".format(fg))
    # Load SGR code associated with desired background color
    if isinstance(bg, (str, type(""))):
        try:
            sgr_code = getattr(ANSI, str('sgr_bg_{}'.format(bg)))
        except AttributeError:
            raise ValueError("incorrect background color: {!r}".format(bg))
        else:
            sgr_list.append(sgr_code)
    elif isinstance(bg, tuple):
        sgr_code = ANSI.sgr_bg_rgb(*bg)
        sgr_list.append(sgr_code)
    elif isinstance(bg, int):
        sgr_code = ANSI.sgr_bg_indexed(bg)
        sgr_list.append(sgr_code)
    elif bg is None:
        pass
    else:
        raise ValueError("incorrect background color: {!r}".format(bg))
    # Load single SGR code for "style"
    if style is not None:
        try:
            sgr_code = getattr(ANSI, str('sgr_{}'.format(style)))
        except AttributeError:
            raise ValueError("incorrect text style: {!r}".format(style))
        else:
            sgr_list.append(sgr_code)
    # Load additional SGR codes (custom)
    for name, active in sgr.items():
        try:
            sgr_code = getattr(ANSI, str('sgr_{}'.format(name)))
        except AttributeError:
            raise ValueError("incorrect custom SGR code: {!r}".format(name))
        else:
            if active:
                sgr_list.append(sgr_code)
    # Combine everything into one sequence
    if reset:
        return ANSI.cmd_sgr(sgr_list) + text + ANSI.cmd_sgr_reset_all
    else:
        return ANSI.cmd_sgr(sgr_list) + text


class ANSIFormatter(object):

    """
    Formatter for ANSI Set Graphics Rendition codes.

    An instance of this class is inserted into the context object as `ansi`.
    Using the fact that `ANSIFormatter` is callable one can easily add
    ANSI control sequences for foreground and background color as well as text
    attributes.
    """

    def __init__(self, enabled=None):
        """
        Initialize an ANSI Formatter.

        :param enabled:
            A tri-state that controls the formatter.  If `enabled` is True or
            False then the obvious meaning is assumed. If `enabled` is None
            then the effective value is computed using ``sys.stdout.isatty()``.
        """
        if enabled is None:
            enabled = sys.stdout.isatty()
        self._enabled = enabled

    @property
    def is_enabled(self):
        """
        Flag indicating if text style is enabled.

        This property is useful to let applications customize their
        behavior if they know color support is desired and enabled.
        """
        return self._enabled

    def cmd(self, cmd, *args):
        """Get an ANSI control sequence, if the formatter is enabled."""
        if self._enabled:
            return ansi_cmd(cmd, *args)
        else:
            return ''

    def __call__(self, text, fg=None, bg=None, style=None, reset=True, **sgr):
        """
        Format given text with ANSI control codes.

        If the formatter is enabled this is a pass-through to
        :func:`ansi_sgr()`. Otherwise this is a no-op that returns ``text``.
        """
        if fg == 'auto' and bg is not None:
            fg = get_visible_color(bg)
        elif bg == 'auto' and fg is not None:
            bg = get_visible_color(fg)
        if self._enabled:
            return ansi_sgr(text, fg, bg, style, reset, **sgr)
        else:
            return text

    available_colors = (
        str('black'), str('red'), str('green'), str('blue'), str('magenta'),
        str('cyan'), str('white'))

    available_bright_colors = (
        str('bright_black'), str('bright_red'), str('bright_green'),
        str('bright_blue'), str('bright_magenta'), str('bright_cyan'),
        str('bright_white'))

    available_styles = (
        None, 'bold', 'dim', 'italic', 'underline', 'blink_slow', 'blink_fast',
        'reverse', 'concealed', 'crossed'
    )

    def _aprint2(self, *values, **kwargs):
        """
        ANSI formatting-aware print().

        This method is a version of print() (function) that understands
        additional ansi control parameters.

        :param value:
            The values to print, same as with ``print()``
        :param sep:
            Separator between values, same as with ``print()``
        :param end:
            Terminator of the line, same as with ``print()``
        :param file:
            File to print to, same as with ``print()``
        :param flush:
            Flag that controls stream flush behavior, same as with ``print()``
        :param fg:
            Foreground color, same as with :meth:`__call__()`.
        :param bg:
            Background color, same as with :meth:`__call__()`.
        :param style:
            Text style, same as with :meth:`__call__()`.
        :param reset:
            Flag that controls if ANSI attributes are reset at the end, same as
            with :meth:`__call__()`.
        :param sgr:
            Additonal (custom) Set Graphics Rendition directives, same as with
            :meth:`__call__()`.

        .. note::
            This implementation is intended for Python 2
        """
        sep = kwargs.pop(str('sep'), ' ')
        end = kwargs.pop(str('end'), '\n')
        file = kwargs.pop(str('file')) or sys.stdout
        flush = kwargs.pop(str('flush'), False)
        fg = kwargs.pop(str('fg'), None)
        bg = kwargs.pop(str('bg'), None)
        style = kwargs.pop(str('style'), None)
        reset = kwargs.pop(str('reset'), True)
        sgr = kwargs
        text = sep.join(str(value) for value in values)
        text = self(text, fg, bg, style, reset, **sgr)
        print(text, end=end, file=file)
        if flush:
            file.flush()

    def _aprint3(self, *values, **kwargs):
        """
        ANSI formatting-aware print().

        This method is a version of print() (function) that understands
        additional ansi control parameters.

        :param value:
            The values to print, same as with ``print()``
        :param sep:
            Separator between values, same as with ``print()``
        :param end:
            Terminator of the line, same as with ``print()``
        :param file:
            File to print to, same as with ``print()``
        :param flush:
            Flag that controls stream flush behavior, same as with ``print()``
        :param fg:
            Foreground color, same as with :meth:`__call__()`.
        :param bg:
            Background color, same as with :meth:`__call__()`.
        :param style:
            Text style, same as with :meth:`__call__()`.
        :param reset:
            Flag that controls if ANSI attributes are reset at the end, same as
            with :meth:`__call__()`.
        :param sgr:
            Additonal (custom) Set Graphics Rendition directives, same as with
            :meth:`__call__()`.

        .. note::
            This implementation only works on Python 3
        """
        sep = kwargs.pop('sep', ' ')
        end = kwargs.pop('end', '\n')
        file = kwargs.pop('file') or sys.stdout
        flush = kwargs.pop('flush', False)
        fg = kwargs.pop('fg', None)
        bg = kwargs.pop('bg', None)
        style = kwargs.pop('style', None)
        reset = kwargs.pop('reset', True)
        sgr = kwargs
        text = sep.join(str(value) for value in values)
        text = self(text, fg, bg, style, reset, **sgr)
        print(text, end=end, file=file)
        # NOTE: Don't use print(..., flush=flush) as that doesn't work on
        # Python 3.2. This was https://github.com/zyga/guacamole/issues/9
        if flush:
            file.flush()

    if sys.version_info[0] == 2:
        aprint = _aprint2
    else:
        aprint = _aprint3


class ANSIIngredient(Ingredient):

    """Ingredient for colorizing output."""

    def __init__(self, enable=None):
        """
        Initialize the ANSI ingredient.

        :param enable:
            Tri-state flag that controls if the embedded ANSI formatter
            should be enabled. See :meth:`ANSI.__init__()`.
        """
        if sys.platform == 'win32':
            try:
                import colorama
            except ImportError:
                enable = False
            else:
                colorama.init()
        self._enable = enable

    def added(self, context):
        """Ingredient method called before anything else."""
        context.ansi = ANSIFormatter(self._enable)
        context.aprint = context.ansi.aprint



# encoding: utf-8
# This file is part of Guacamole.
#
# Copyright 2012-2015 Canonical Ltd.
# Written by:
#   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
#
# Guacamole is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3,
# as published by the Free Software Foundation.
#
# Guacamole is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Guacamole.  If not, see <http://www.gnu.org/licenses/>.

"""
Guacamole -- command line applications that suck less.

Guacamole is a flexible, modular system for creating command line applications.
Guacamole comes with built-in support for writing command line applications
that integrate well with the running system. A short list of supported features
(ingredients) includes:

    - handling flat and hierarchical commands
    - hassle-free crash detection
    - hassle-free logging
    - internationalization and localization

The guacamole ingredient system allows for third party add-ons. Please read the
add-on developer guide for details.

.. note::
    Guacamole supports Python 2.7 and Python 3.2, 3.4 and 3.5. Other versions
    are not tested extensively. Versions earlier than 2.7 are not supported and
    won't be. Applications that still need to support the end-of-life Python
    2.x release series are encouraged to update to Python 2.7.

Lower-level classes can be found in the :mod:`guacamole.core` and
:mod:`guacamole.recipes` modules. Add-on developers should use those modules
exclusively. All other APIs are considered private.
"""

from __future__ import absolute_import, print_function

from guacamole.recipes.cmd import Command

__all__ = ('Command',)

__version__ = '0.9.1'



# encoding: utf-8
# This file is part of Guacamole.
#
# Copyright 2012-2015 Canonical Ltd.
# Written by:
#   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
#
# Guacamole is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3,
# as published by the Free Software Foundation.
#
# Guacamole is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Guacamole.  If not, see <http://www.gnu.org/licenses/>.

"""
APIs for guacamole add-on developers.

This module contains the public APIs for add-on developers. Add-ons (or
plug-ins) for guacamole are called **ingredients**. The :class:`Ingredient`
class contains a description of available add-on methods.

Ingredients are somewhat similar to Django middleware as they can influence the
execution of an application across its life-cycle. All of core guacamole
features are implemented as ingredients. Developers are encouraged to read core
ingredients to understand how to formulate their own design.

Ingredient APIs are *public*. They will be maintained for backwards
compatibility. Since Guacamole doesn't automatically enable any third-party
ingredients, application developers that wish to use them need to use the
:mod:`guacamole.core` module to create their own guacamole out of available
ingredients. Ingredient developers are recommended in documenting how to use
each ingredient this way.

In addition this module contains the public APIs for creating custom mixes of
guacamole.  A custom mix begins with a :class:`~guacamole.core.Bowl` with any
number of :class:`~guacamole.core.Ingredient` objects added.

If you are familiar with the :class:`~guacamole.recipes.cmd.Command` class you
should know that they are using the recipe system internally. They refer to
pre-made recipes that put particular ingredients into the bowl for a ready
dish.

If you wish to build a custom experience on top of guacamole, please provide a
new recipe class. Recipes are how applications should interact with any
guacamole mixtures.
"""

from __future__ import absolute_import, print_function, unicode_literals

from guacamole.core import Bowl


__all__ = (
    'Recipe',
    'RecipeError',
)


class Recipe(object):

    """Mechanism to use ingredients to dispatch and invoke commands."""

    def get_ingredients(self):
        """
        Get a list of ingredients for making guacamole.

        :returns:
            A list of initialized ingredients.
        :raises RecipeError:
            If the recipe is wrong. This is a developer error. Do not handle
            this exception. Consult the error message to understand what the
            problem is and correct the recipe instead.
        """

    def prepare(self):
        """
        Prepare a bowl with the ingredients specified by this recipe.

        :return:
            A new :class:`Bowl` instance with all the ingredients prepared.
        """
        return Bowl(self.get_ingredients())

    def main(self, argv=None, exit=True):
        """
        Shortcut to prepare a bowl of guacamole and eat it.

        :param argv:
            Command line arguments or None. None means that sys.argv is used
        :param exit:
            Raise SystemExit after finishing execution
        :returns:
            Whatever is returned by the eating the guacamole.
        :raises:
            Whatever is raised by eating the guacamole.

        .. note::
            This method always either raises and exception or returns an
            object. The way it behaves depends on the value of the `exit`
            argument.

        This method can be used to quickly take a recipe, prepare the guacamole
        and eat it. It is named main as it is applicable as the main method of
        an application.

        The `exit` argument controls if main returns normally or raises
        SystemExit.  By default it will raise SystemExit (it will either wrap
        the return value with SystemExit or re-raise the SystemExit exception
        again). If SystemExit is raised but `exit` is False the argument to
        SystemExit is unwrapped and returned instead.
        """
        bowl = self.prepare()
        try:
            retval = bowl.eat(argv)
        except SystemExit as exc:
            if exit:
                raise
            else:
                return exc.args[0]
        else:
            if retval is None:
                retval = 0
            if exit:
                raise SystemExit(retval)
            else:
                return retval


class RecipeError(Exception):

    """
    Exception raised when the recipe for guacamole is incorrect.

    This exception is only used when a set of ingredients is ordered correctly
    or has some missing elements. Each time this exception is raised it is
    accompanied by a detailed message that should help you to resolve the
    problem.

    .. note::
        This exception should not be handled, it is a developer error.
    """
